# BitkitRobit API (RUS)

## `class BitkitRobit`

Класс `BitkitRobit` для управления манипулятором Робiт.

### `BitkitRobit()`

Создает новый объект BitkitRobit, который включает модули управления:

- 4× сервопривода
- 4× датчика угла поворота (энкодера)
- 4× тактовые кнопки
- 2× светодиода

Пины управления перечисленных компонентов прописаны в библиотеки и не меняются, т.к. модули на BitKit Shield’е связаны электрической связью только с определенными контактными пинами.

### `void begin(uint8_t value1, uint8_t value2, uint8_t value3, uint8_t value4)`

Инициализирует манипулятор:

- Назначает пины светодиодов в режим выхода.
- Назначает пины кнопок в режим входа с программной подтяжкой к питанию.
- Назначаем пины энкодеров в режим входа с программной подтяжкой к питанию.
- Устанавливает начальное положение энкодеров.
- Включает сервоприводы в режиме плавного подключения.
- Очищает всю память от сохранённых положений манипулятора.

#### Аргументы

- `value1`: стартовая позиция сервопривода №1. При отсутствии принимается равным 45.
- `value2`: стартовая позиция сервопривода №2. При отсутствии принимается равным 90.
- `value3`: стартовая позиция сервопривода №3. При отсутствии принимается равным 90.
- `value4`: стартовая позиция сервопривода №4. При отсутствии принимается равным 90.

Вызовите этот метод до начала взаимодействия с BitkitRobit. Например в функции `setup()`.

#### Возвращаемые значения

Нет

### `void digitalWrite(ManipulatorLed led, bool value)`

Включает или выключает один из встроенных светодиодов на BitKit Shield’е.

#### Аргументы

- `led`: имя светодиода. Допустимые значения: `ManipulatorLed::LED1` и `ManipulatorLed::LED2`.
- `value`: определяет желаемое состояние светодиода. Допустимые значения: `HIGH` / `1` / `true` для включения или `LOW` / `0` / `false` для выключения.

#### Возвращаемые значения

Нет

### `bool digitalRead(ManipulatorButton sw)`

Считывает значения одной из встроенных кнопок на BitKit Shield’е.

#### Аргументы

Аргумент `sw` имя считываемой кнопки. Допустимые значения:

- `ManipulatorButton::SW1`: отмечена SW1 на плате.
- `ManipulatorButton::SW2`: отмечена SW2 на плате.
- `ManipulatorButton::SW3`: отмечена SW3 на плате.
- `ManipulatorButton::SW4`: отмечена SW4 на плате.

#### Возвращаемые значения

Возвращает `false` если кнопка нажата, в противном случае `true`.

### `void setStep(uint8_t step1, uint8_t step2, uint8_t step3, uint8_t step4)`

Устанавливает отношения шага каждого энкодера к углу поворота сервопривода.

#### Аргументы

- `step1`: чувствительность энкодера ENC1.
- `step2`: чувствительность энкодера ENC2.
- `step3`: чувствительность энкодера ENC3.
- `step4`: чувствительность энкодера ENC4.

#### Возвращаемые значения

Нет

### `void getStep(uint8_t& step1, uint8_t& step2, uint8_t& step3, uint8_t& step4)`

Возвращает в аргументы отношения шага каждого энкодера к углу поворота сервопривода.

#### Аргументы

- `step1`: чувствительность энкодера ENC1.
- `step2`: чувствительность энкодера ENC2.
- `step3`: чувствительность энкодера ENC3.
- `step4`: чувствительность энкодера ENC4.

### `void setRange(uint8_t minValue1, uint8_t maxValue1, uint8_t minValue2, uint8_t maxValue2, uint8_t minValue3, uint8_t maxValue3, uint8_t minValue4, uint8_t maxValue4)`

Устанавливает диапазон допустимого угла поворота каждого из сервоприводов.

#### Аргументы

- `minValue1`: минимальный угол сервопривода №1.
- `maxValue1`: максимальный угол сервопривода №1.
- `minValue2`: минимальный угол сервопривода №2.
- `maxValue2`: максимальный угол сервопривода №2.
- `minValue3`: минимальный угол сервопривода №3.
- `maxValue3`: максимальный угол сервопривода №3.
- `minValue4`: минимальный угол сервопривода №4.
- `maxValue4`: максимальный угол сервопривода №4.

#### Возвращаемые значения

Нет

### `void getRange(uint8_t& minValue1, uint8_t& maxValue1, uint8_t& minValue2, uint8_t& maxValue2, uint8_t& minValue3, uint8_t& maxValue3, uint8_t& minValue4, uint8_t& maxValue4)`

Возвращает в аргументы диапазон допустимого угла поворота каждого из сервоприводов.

#### Аргументы

- `minValue1`: минимальный угол сервопривода №1.
- `maxValue1`: максимальный угол сервопривода №1.
- `minValue2`: минимальный угол сервопривода №2.
- `maxValue2`: максимальный угол сервопривода №2.
- `minValue3`: минимальный угол сервопривода №3.
- `maxValue3`: максимальный угол сервопривода №3.
- `minValue4`: минимальный угол сервопривода №4.
- `maxValue4`: максимальный угол сервопривода №4.

#### Возвращаемые значения

Нет

### `void setPosition(uint8_t value1, uint8_t value2, uint8_t value3, uint8_t value4)`

Устанавливает манипулятор в заданное положение. Если значения не входит в диапазон допустимых углов из функции `getRange`, то берется максимальный или минимальный угол границы диапазона соответственно.

#### Аргументы

- `value1`: угол сервопривода №1. Диапазон от 0 до 180 градусов.
- `value2`: угол сервопривода №2. Диапазон от 0 до 180 градусов.
- `value3`: угол сервопривода №3. Диапазон от 0 до 180 градусов.
- `value4`: угол сервопривода №4. Диапазон от 0 до 180 градусов.

#### Возвращаемые значения

Нет

### `void getPosition(uint8_t& value1, uint8_t& value2, uint8_t& value3, uint8_t& value4)`

Возвращает в аргументы значения с текущего положения манипулятора.

#### Аргументы

- `value1`: угол сервопривода №1.
- `value2`: угол сервопривода №2.
- `value3`: угол сервопривода №3.
- `value4`: угол сервопривода №4.

#### Возвращаемые значения

Нет

### `void savePosition(ManipulatorPositionNumber positionNumber)`

Сохраняет текущее положение манипулятора в память микроконтроллера.

#### Аргументы

- `positionNumber`: номер ячейки для сохранения положения манипулятора. Допустимые значения
  `POSITIONx`, где x цифра от 0 до 10.

#### Возвращаемые значения

Нет

### `void loadPosition(ManipulatorPositionNumber positionNumber)`

Загружает текущее положение манипулятора из памяти микроконтроллера.

#### Аргументы

- `positionNumber`: номер ячейки для загрузки положения манипулятора. Допустимые значения `POSITIONx`, где x цифра от 0 до 10.

#### Возвращаемые значения

Нет

### `void clearPositions()`

Очищает все ячейки памяти от положения манипулятора.

#### Аргументы

Нет

#### Возвращаемые значения

Нет

### `void update()`

Обновляет положения манипулятора в соответствии с поворотом ручек энкодеров. Вызывайте этот метод как можно чаще в функции `loop()`.

#### Аргументы

Нет

#### Возвращаемые значения

Нет